<?php
/*
* GamePHP: Open-source library for game development
* Copyright (C) 2005 Jawad Stouli <jawad.xos@gmail.com>
* 
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

/**
 * Action
 *
 * @package Engine
 * @author Jawad <jawad.xos@gmail.com>
 * @copyright Copyright (C) 2005 <a href="http://gamephp.sourceforge.net/">GamePHP</a>
 * @license http://opensource.org/licenses/gpl-license.php GNU Public License
 */

if(!_GAMEPHP_ENGINE) die("Main file of GamePHP (Engine.php) required.");

/**
 * Action
 *
 * @package Engine
 * @example Actions/TheoricalExample.php
 * @author Jawad <jawad.xos@gmail.com>
 */
final class Action {
	/**
	 * Internal reference on action's object
	 */
	protected $action;

	/**
	 * Constructor
	 *
	 * @param String Action to load
	 */
	public function __construct($aName) {
		if(!file_exists(Engine::file("actions/".$aName))) return Warning(1001, "__construct", __FILE__, __LINE__, array("\$aName" => $aName));
		
		$this->action = $aName;
	}

	/**
	 * Do action
	 *
	 * @param Entity Entity doing the action (instance of Entity -or any children class-)
	 * @param Array Array of parameters (depends of the action; see Actions/ActionName.php).
	 *
	 * @since 0.1
	 * @example Actions/Move.php
	 *
	 * @return Result A Result object (see Result class)
	 */
	public function act(Entity &$Entity, $params) {		
		$Actions =& $GLOBALS["GamePHP"]["Actions"];
		
		require Engine::file("actions/".$this->action);
		
		// We check that the entity is able to do this action
		if($Actions[$this->action]["entities"] != _GAMEPHP_ACTIONS_ALL && !in_array(get_class($Entity), $Actions[$this->action]["entities"])) return Warning(1002, "act", __FILE__, __LINE__, array("Entity \$Entity" => $Entity, "\$params" => $params));
		
		// We check that given parameters fit needed ones
		foreach($Actions[$this->action]["params"] as $key=>$value) {
			if(!array_key_exists($value, $params)) return Warning(1003, "do", __FILE_, __LINE__, array("Entity \$Entity" => $Entity, "\$params" => $params, "needed_params" => $Actions[$this->action]["params"]));
			
			$validparams[$value] =& $params[$value];
		}
		
		$funcName = $this->action;
		$ext = Engine::gamefile("actions/my".$funcName);
		
		// Loading action's extension (file prefixed by 'my' and located in gamedir/actions/)
		if(file_exists($ext)) require_once $ext;
		
		// Executing action
		$a = $funcName($Entity, $validparams);
		
		// Dispatching action's event
		if(isSuccessful($a)) EventManager::dispatchEvent("Entity", "Action", array("Entity" => $Entity, "ActionName" => $this->action, "ActionParams" => $params));
		
		return $a;
	}
}

/*
 * All entities can use an action (constant to be used as a value for $Actions["ActionName"]["entities"]).
 */
define("_GAMEPHP_ACTIONS_ALL", 0);
?>