<?php
/*
* GamePHP: Open-source library for game development
* Copyright (C) 2005 Jawad Stouli <jawad.xos@gmail.com>
* 
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

/**
 * Classe Entity
 *
 * @see Player.php
 *
 * @package Engine
 * @author Jawad <jawad.xos@gmail.com>
 */
 
if(!_ENGINE) die("Fichier principal de GamePHP non-charg.");

/**
 * Fonctions d'accs  la base de donnes.
 */
require_once Engine::file("utils/SpecModel");

/**
 * Classe Entity
 *
 * Cette classe sert de base  la gestion des entits.
 *
 * @package Engine
 * @author Jawad <jawad.xos@gmail.com>
 */
class Entity extends SpecModel {
	/**
	 * Champs et valeurs
	 */
	protected $fields = array();
	
	/**
	 * Champ ID
	 */
	protected $fieldid;
	
	/**
	 * Rollback
	 */
	protected $rollback;
	
	/**
	 * Champs mis  jour ( mettre  jour dans la base de donnes lors de l'appel de update()
	 */
	protected $updated = array();
	
	/**
	 * Liste des types d'entits existantes
	 */
	static $list;
	
	/**
	 * Table de la base de donnes associe  cette entit
	 */
	public $table;
	
	/**
	 * Constructeur
	 *
	 * @param Int Identifiant des donnes  utiliser dans la table (champs id)
	 */
	public function __construct($id) {
		$this->table = getEntityTable($this);
		$this->fieldid = DBUtils::getField("id", $this->table);
		
		$this->fields = getConnection()->query("SELECT * FROM {table} WHERE ".$this->fieldid."='".$id."' LIMIT 0,1", $this->table)->fetch();
		
		$objfield = DBUtils::getField("objects", $this->table);
		if(isset($this->fields[$objfield])) {
			if(!empty($this->fields[$objfield])) $this->fields[$objfield] = unserialize($this->fields[$objfield]);
			else $this->fields[$objfield] = Array();
		}
		
		unset($objfield);		
	}
	
	/**
	 * Surcharge de l'accs aux variables de la classe.
	 *
	 * @param String Variable demande
	 *
	 * @link http://www.php.net/overload
	 *
	 * @return Mixed
	 */
	public function __get($name) {
    	if($name == "caracs") {
    		return $this->fields;
		} elseif(isset($this->fields[$name])) {
        	return $this->fields[$name];
    	}
	}
		
	/**
	 * Annuler la dernire mise  jour des donnes.
	 *
	 * Cette fonction vous permet d'annuler les oprations effectues par la fonction update().
	 * A n'utiliser qu'aprs l'appel de update().
	 *
	 * @return Mixed
	 */
	public function rollback() {
		$rs = getConnection()->update($this->rollback, array($this->fieldid => $this->fields[$this->fieldid]), $this->table, true);
    	
    	if(!$rs) {
    		return $rs;
    	} else {
    		unset($this->rollback);
    		
    		return true;
    	}
	}
  
	/**
	 * Surcharge de l'criture des variables de la classe.
	 *
	 * @param String Variable demande
	 * @param Mixed Valeur
	 *
	 * @link http://www.php.net/overload
	 *
	 * @return Mixed
	 */
	public function __set($name, $value) {
		//echo "Set::{$name} = {$value}";
		
    	if(isset($this->fields[$name]) && $this->fields[$name] != $value) {
    		$this->updated[$name] = $this->fields[$name];
			$this->fields[$name] = $value;
		} 
    }

	/**
	 * Mettre  jour la base de donnes.
	 *
	 * Cette fonction met  jour la base de donnes en fonction des modification qui ont t faites sur les valeurs.
	 * 
	 * @return Bool
	 */
    public function update() {
    	if(count($this->updated) <= 0) return true; // On vrifie qu'il y'a bien eu des modifications
    	
    	$values = array();
    
    	foreach($this->updated as $field=>$value) {
    		$values[$field] = $this->fields[$field]; 
    	}
    	
    	//On envoie l'vnement Update
    	EventManager::dispatchEvent("Entity", "Update", array("Entity" => $this, "updated" => $values));
    	
    	$rs = getConnection()->update($values, array($this->fieldid => $this->fields[$this->fieldid]), $this->table, true);
    	
    	if(!$rs) {
    		return false;
    	} else {
    		// On prpare le tableau ncessaire  un ventuel rollback
    		$this->rollback = $this->updated;
    	
    		$this->updated = array();
    		
    		// On envoie l'vnement Updated
    		EventManager::dispatchEvent("Entity", "Updated", array("Entity" => $this));
    		
    		return true;
    	}
    }
    
    /**
     * Vrifier si une valeur a t mise  jour lors du dernier appel  Update
     *
     * @param $var String Nom de la variable  vrifier
     *
     * @return Bool
     */
    public function updated($var) {
    	return array_key_exists($var, $this->rollback);
    }
}

/**
 * Permet d'enregistrer un nouveau type d'entit
 *
 * Cette fonction doit tre appele aprs la dfinition de la classe $eName.
 * La liste des entits enregistres est utile lorsque le moteur doit traiter des fonctions utilisant tous les types d'entits.
 * Exemple de fonction: Map::getEntitiesAround
 *
 * @param String $eName
 * @param String $eTable Identifiant de la table associe  l'entit
 *
 * @return Void
 */
function registerEntity($eName, $eTable) {	
	if(class_exists($eName)) Entity::$list[$eName] = $eTable;
}

/**
 * Rcuprer la liste des entits enregistres
 *
 * @return Array
 */
function getEntityList() {
	return Entity::$list;
}

/**
 * Rcuprer l'identifiant de la table contenant les instances d'entit
 *
 * @param String $eName Nom de l'entit
 *
 * @return String
 */
function getEntityTable(&$entity) {
	$eName = get_class($entity);

	return Entity::$list[$eName];
}

// On ajoute le type d'vnement "Entity" et quelques vnements
EventManager::registerType("Entity");
EventManager::registerEvent("Entity", "Action");
EventManager::registerEvent("Entity", "Update");
EventManager::registerEvent("Entity", "Updated");
?>