<?php
/*
* GamePHP: Open-source library for game development
* Copyright (C) 2005 Jawad Stouli <jawad.xos@gmail.com>
* 
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

/**
 * Gestionnaire d'vnements
 *
 * @see Events.php
 *
 * @package Engine
 * @author Jawad <jawad.xos@gmail.com>
 */
 
/**
 * Classe charge de la gestion des vnements (ajout et transmission)
 *
 * @package Engine
 * @author Jawad <jawad.xos@gmail.com>
 */
abstract class EventManager {
	/**
	 * Types d'vnements disponibles
	 */
	static $Types = array();
	
	/**
	 * Ecouteurs (par type) enregistrs
	 */
	static $Listeners = array();
	
	/**
	 * Ecouteurs (par vnement) enregistrs
	 */
	static $EListeners = array();
	
	/**
	 * Evnements disponibles
	 */
	static $Events = array();
	
	/**
	 * Ajouter un couteur pour un type
	 *
	 * Un couteur ajout avec cette fonction sera appel lorsque n'importe quel vnement du type est transmis.
	 *
	 * @param $type String Type d'couteur
	 * @param $callback Array Fonction  appeler ([f] => nom de la fonction,  [o] => objet contenant la fonction)
	 *
	 * @return Bool
	 */ 
	public function addListener($type, $callback) {
		$Types = (array)self::$Types;
		
		if(in_array($type, $Types) && is_array($callback)) {
			self::$Listeners[$type][] = $callback;	
			
			return true;
		} else {
			return false;	
		}
	}
	
	/**
	 * Ajouter un couteur pour un vnement
	 *
	 * Un couteur ajout avec cette fonction sera appel uniquement lorsqu'un vnement prcis est transmis.
	 *
	 * @param $type String Type d'couteur
	 * @param $event String Evnement transmis
	 * @param $callback Array Fonction  appeler ([f] => nom de la fonction,  [o] => objet contenant la fonction)
	 *
	 * @return Bool
	 */
	public function addEventListener($type, $event, $callback) {
		if(is_array($callback) && in_array($type, self::$Types) && in_array($event, self::$Events[$type])) {
			self::$EListeners[$type][$event][] = $callback;
			
			return true;
		} else {
			return false;	
		}
	}
	
	/**
	 * Transmettre un vnement aux couteurs enregistrs
	 *
	 * @param $type String Type de l'vnement
	 * @param $event String Identifiant de l'vnement
	 * @param $params Array Tableau de paramtres
	 *
	 * @return Void
	 */
	public function dispatchEvent($type, $event, $params) {
		if(!in_array($type, self::$Types) || !in_array($event, self::$Events[$type])) return false;
		
		$Listeners = (array)self::$Listeners[$type];
		$EListeners = (array)self::$EListeners[$type][$event];
		
		// On traite les couteurs gnraux
		foreach($Listeners as $eCallback) {
			if(isset($eCallback["o"])) {
				$eCallback["o"]->$eCallback["f"]($event, $params);	
			} else {
				$eCallback["f"]($event, $params);
			}
		}
		
		// On traite les couteurs spcifiques
		foreach($EListeners as $eCallback) {
			if(isset($eCallback["o"])) {
				$eCallback["o"]->$eCallback["f"]($params);
			} else {
				$eCallback["f"]($params);
			}
		}
	}
	
	/**
	 * Enregistrer un type d'vnements
	 *
	 * @param $type String Nom du type
	 *
	 * @return Void
	 */
	public function registerType($type) {
		self::$Types[] = $type;
	}
	
	/**
	 * Enregistrer un vnement pour un type
	 *
	 * @param $type String Nom du type (pralablement enregistr avec registerType)
	 * @param $event String Nom de l'vnement
	 *
	 * @return Bool
	 */
	public function registerEvent($type, $event) {
		if(in_array($type, self::$Types)) self::$Events[$type][] = $event;
		else return false;	
	}
}
?>