<?php
/*
* GamePHP: Open-source library for game development
* Copyright (C) 2005 Jawad Stouli <jawad.xos@gmail.com>
* 
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

/**
 * Zone d'une carte
 *
 * @package Engine
 * @author Jawad <jawad.xos@gmail.com>
 */

/**
 * Zone d'une carte
 *
 * Permet de travailler sur une certaine zone d'une carte afin de ne pas surcharger le serveur en requtes.
 *
 * @package Engine
 * @subpackage Maps
 * @author Jawad <jawad.xos@gmail.com>
 */
interface MapZone {
	/**
	 * Crer une zone ( ne pas appeler directement, utiliser Map::getZone
	 *
	 * @param Map $map Carte contenant la zone
	 * @param Array $coords Tableau de coordonnes
	 *
	 * @return MapZone
	 */
	public function __construct(Map $map, $coords);
	
	/**
	 * Vrifier qu'une position appartient bien  la zone
	 *
	 * @param Array $coords Tableau de coordonnes
	 *
	 * @return Bool
	 */
	public function positionIn($coords);
	
	/**
	 * Obtenir les calques d'une position (champ `layers` de la table map)
	 *
	 * @param Array $coords Tableau de coordonnes
	 *
	 * @return Array
	 */
	public function getPositionLayers($coords);
	
	/**
	 * Dfinir les calques d'une position (n'intervient pas sur la base de donnes!)
	 *
	 * Voir Map::setPositionLayer pour dfinir les calques d'une position dans la base de donnes.
	 *
	 * @param Array $coords Tableau de coordonnes
	 * @param Array $layers Calques
	 *
	 * @return Void
	 */
	public function setPosition($coords, $layers);
	
	/**
	 * Dfinir la position d'une entit
	 *
	 * @param Array $coords Tableau de coordonnes
	 * @param Int Identifiant de l'entit
	 * @param String Image
	 *
	 * @return Void
	 */
	public function setPositionEntity($coords, $id, $image);
}

/**
 * Itrateur de zone
 *
 * Permet d'effectuer une boucle sur tous les couples x/y d'une zone, et de rcuprer les calques associs
 *
 * @package Engine
 * @subpackage Maps
 * @author Jawad <jawad.xos@gmail.com>
 */
class ZoneIterator extends MyIterator {
	/**
	 * Zone
	 */
	private $zone;
	
	/**
	 * Constructeur
	 *
	 * @param MapZone $zone Zone
	 *
	 * @return ZoneIterator
	 */
	public function __construct(MapZone $zone) {
		$this->zone =& $zone;
		
		$x1 = ($zone->xorder == "ASC") ? $zone->x[1] : $zone->x[2];
		$x2 = ($zone->xorder == "ASC") ? $zone->x[2] : $zone->x[1];
		$y1 = ($zone->yorder == "ASC") ? $zone->y[1] : $zone->y[2];
		$y2 = ($zone->yorder == "ASC") ? $zone->y[2] : $zone->y[1];
		
		$this->xorder = $zone->xorder;
		$this->yorder = $zone->yorder;
		
		parent::__construct(array("xmin" => $x1, "xmax" => $x2, "ymin" => $y1, "ymax" => $y2));
	}
	
	/**
	 * Construire le tableau de rsultat
	 *
	 * @return Array
	 */
	protected function buildResult() {
		return array("x" => $this->currx, "y" => $this->curry, "layers" => $this->zone->getPositionLayers(array("x" => $this->currx, "y" => $this->curry), true));
	}
	
	/**
	 * Obtenir les calques d'une position, et avancer  la position suivante (automatiquement)
	 *
	 * @return Array
	 */
	public function fetchPosition() {
		return parent::fetch();
	}
}
?>