<?php
/*
* GamePHP: Open-source library for game development
* Copyright (C) 2005 Jawad Stouli <jawad.xos@gmail.com>
* 
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

/**
 * Gestionnaire d'objets
 *
 * @package Engine
 * @author Jawad <jawad.xos@gmail.com>
 */

/**
 * On inclut l'objet "Object" (classe de base)
 */
require_once Engine::file("Objects/Object");

/**
 * Instancier un objet
 *
 * @param Int $objectid Identifiant unique de l'objet dans la table `objects` (interne)
 * 
 * @return Object
 */
function &getObject($objectid) {
	$oCaracs = infoObject($objectid);
	$oCaracs["caracs"] = unserialize($oCaracs["caracs"]);
	
	if(!file_exists(Engine::file("objects/".$oCaracs["type"])))	return Warning(3001, "&getObject", __FILE_, __LINE__, array("Caractristiques de l'objet" => $oCaracs));
		
	require_once Engine::file("objects/".$oCaracs["type"]);
		
	return new $oCaracs["type"]($oCaracs);
}

/**
 * Obtenir des informations sur un objet
 *
 * @param Int $objectid Identifiant unique de l'objet dans la table `objects` (interne)
 * 
 * @return Object
 */
function infoObject($objectid) {
	return getConnection()->get("*", "objects", array("id" => $objectid));
}

/**
 * Obtenir une liste des objets d'une entit
 *
 * @param Entity $Entity Entit
 *
 * @return Array
 */
function listObjects(Entity $Entity) {
	$objfield = DBUtils::getField("objects", $Entity->table);
	
	$listObjects = Array();

	foreach($Entity->{$objfield} as $key=>$value) {
		if(!array_key_exists($value, $listObjects)) {
			$tmp = infoObject($value);
			
			$listObjects[$value]["name"] = $tmp["name"];
			$listObjects[$value]["desc"] = $tmp["desc"];
			$listObjects[$value]["type"] = $tmp["type"];
			$listObjects[$value]["qty"] = 1;
			
			unset($tmp);
		} else {
			$listObjects[$value]["qty"]++;
		}
	}
	
	return $listObjects;
}

?>