<?php
/*
* GamePHP: Open-source library for game development
* Copyright (C) 2005 Jawad Stouli <jawad.xos@gmail.com>
* 
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

/**
 * Fichier d'action (Move)
 *
 * Cette action se charge de dplacer une entit.
 *
 * @package Engine
 * @subpackage Actions
 */

$GLOBALS["GamePHP"]["Actions"]["Move"]["entities"] = _GAMEPHP_ACTIONS_ALL; // Toutes les entits peuvent effectuer cette action (voir Attack.php -dans le testgame- pour une liste restreinte)
$GLOBALS["GamePHP"]["Actions"]["Move"]["params"] = array("x", "y");
$GLOBALS["GamePHP"]["Actions"]["Move"]["errors"] = array(
	0 => "La place demande est dj occupe.",
	1 => "Un problme technique empche votre dplacement. Veuillez ressayer plus tard.",
	2 => "Dplacement effectu avec succs.",
	3 => "Vous n'tes pas en position d'effectuer ce dplacement.",
);

/**
 * Dplacer une entit
 *
 * @param Entity Entit  dplacer
 * @param Array Paramtres de dplacement - array("x" => nouveau x, "y" => nouveau y, "z" => nouveau z) -
 *
 * @example Actions/Move.php
 *
 * @return Int 0 (La place est occupe), 1 (Mise  jour des donnes impossible), 2 (Dplacement effectu)
 */
function Move(Entity &$Entity, $params) {
	$Actions = $GLOBALS["GamePHP"]["Actions"];
	
	if(caseTest(array("x" => $Entity->x, "y" => $Entity->y), array("x" => $params["x"], "y" => $params["y"])) != 1) return Warning($Actions["Move"]["errors"][3], "Move", __FILE__, __LINE__, array("x" => $Entity->x, "y" => $Entity->y, "params" => $params)); // Les deux cases ne sont pas colles ou sont les mmes; dplacement impossible!

	if(getConnection()->exists($params, $Entity->table)) return Warning($Actions["Move"]["errors"][0], "Move", __FILE__, __LINE__, array("x" => $Entity->x, "y" => $Entity->y, "params" => $params));

	if(function_exists("myMove")) {
		$myMove = myMove($Entity, $params); // Appel d'une possible extension
		
		if(isError($myMove) || isWarning($myMove)) return $myMove;
	}
	
	$Entity->x = $params["x"];
	$Entity->y = $params["y"];
	
	if(!$Entity->update()) return Warning($Actions["Move"]["errors"][1], "Move", __FILE__, __LINE__, array("x" => $Entity->x, "y" => $Entity->y, "params" => $params));
		
	return Success($Actions["Move"]["errors"][2]);
}
?>