<?php
/*
* GamePHP: Open-source library for game development
* Copyright (C) 2005 Jawad Stouli <jawad.xos@gmail.com>
* 
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

/**
* Action: Se dplacer ( adapter pour GamePHP)
*
* @version 0.1
* @package Engine
* @subpackage Actions
*/

/**
* Inclusion de la classe Object
*/
//require Engine::file("Objects/Object", "Utils/Case");

// Entits concernes par cette action
$GLOBALS["GamePHP"]["Actions"]["MoveComplex"]["entities"] = _GAMEPHP_ACTIONS_ALL; // false = toutes
$GLOBALS["GamePHP"]["Actions"]["MoveComplex"]["params"] = array("newpos");

/**
* Faire bouger une entit
* (Tenter de limiter au maximum le nombre de requtes MySQL -comme partout en fait-, 2 pour le moment)
*
* @param Entity $Entity Entit  dplacer
* @param Array $params Paramtres du dplacement [Array("newpos" => tableau contenant la nouvelle position X/Y ou une liste de dplacements)] 
*
* @todo Adaptation
* @return Mixed[Bool,Error]
*/
function MoveComplex(Entity $Entity, $params) {
	$Actions = $GLOBALS["GamePHP"]["Actions"];
	
	$DB =& getConnection();
	
	if(!checkParams($params, $Actions["MoveComplex"]["params"])) return new Result(1001, array("action" => "Move", "params" => $params, "needed" => $Actions["MoveComplex"]["params"]), "Move"); // Paramtres incorrects
	
	$newPos = $params["newpos"];
	
	// Cot d'un dplacement
	$cout = 1;
		    
    if(isset($newPos["x"])) {
    	// On vrifie avant tout que la nouvelle position n'est pas la mme que celle de l'entit/que la nouvelle position y est bien colle
		if(caseTest($newPos, Array("x" => $Entity->x, "y" => $Entity->y)) != 1) return Error(1003, array("dep_type" => "simple", "new_pos" => $newPos, "entity_pos" => array("x" => $Entity->x, "y" => $Entity->y)), "Move");

    	// Un seul dplacement  effectuer (tant mieux!).
    	// On vrifie si la nouvelle case est libre.
    	// A faire: Remplacer par Database::count()
    	$cnt = $DB->query("SELECT COUNT(*) FROM {table} WHERE x='".$newPos["x"]."' AND y='".$newPos["y"]."' AND z='".$Entity->z."' AND subz='".$Entity->subz."'", $Entity->_table)->fetchRow(DB_FETCHMODE_ASSOC);
    	$cnt = $cnt["count(*)"];

    	if($cnt >= 1) {
    		// La place est occupe
    		return Error(1004, array("dep_type" => "simple", "new_pos" => $newPos), "Move");
		}
			
		// On calcule le cot d'un dplacement
		$tDep = 1;
			
		// On vrifie que le perso a assez de mouvements disponibles
		if($tDep > $Entity->t_mvt) return Error(1006, array("dep_type" => "simple", "new_pos" => $newPos, "cout_dep" => $tDep, "mvt_dispo" => $Entity->t_mvt), "Move");
			
		// On dfinit le nouveau X et le nouveau Y du perso
		$newX = $newPos["x"];
		$newY = $newPos["y"];
    } else {
    	$nbMoves = count($newPos); // Nombres de dplacements  effectuer
    
    	// On vrifie qu'il y'a bien plusieurs dplacements  effectuer, si il y'en a un seul dans un tableau mal form, on rappelle la fonction de manire plus intelligente
    	if($nbMoves == 1) return Move($Entity, $newPos[0]);
    
    	// Plusieurs dplacements  effectuer
    	// $newPos est donc un tableau de positions, on va tout d'abord calculer les malus de vents/courants et d'encombrement (si ncessaire) puis tester chaque position avant d'effectuer les dplacements.
    	
    	// On vrifie avant tout que la nouvelle position n'est pas la mme que celle de l'entit/que la nouvelle position y est bien colle
		if(caseTest($newPos[0], Array("x" => $Entity->x, "y" => $Entity->y)) != 1) return Error(1005, array("dep_type" => "multi", "new_pos0" => $newPos[0], "new_pos" => $newPos, "entity_pos" => array("x" => $Entity->x, "y" => $Entity->y)), "Move");
			
		// On connait le nombre de mouvements  effectuer. On rcupre les informations de la base de donnes concernant l'entourage du perso.
		$rs = $DB->query("SELECT `x`, `y` FROM {table} WHERE x >= '".($Entity->x - $Entity->perception)."' AND x <= '".($Entity->x + $Entity->perception)."' AND y >= '".($Entity->y - $Entity->perception)."' AND y <= '".($Entity->y + $Entity->perception)."' AND z='".$Entity->z."' AND subz='".$Entity->subz."'", $Entity->_table);
	
		// On va parcourir les donnes extraites de la base et les mettres dans un tableau
		while($c = $rs->fetchRow()) {
			$tmpPos[$c["x"].$c["y"]] = true;
		}
			
		// On va maintenant parcourir la liste des dplacements  effectuer de l'entit et vrifier que chaque place est libre
		$i = 0;
		while($i <= ($nbMoves - 1)) {
			// On vrifie avant tout que la nouvelle position n'est pas la mme que la position de dpart de l'entit et qu'elle y est colle
    		if($i > 0) {
				if(caseTest($newPos[$i],$newPos[$i - 1]) != 1) return Error(1003, array("new_pos[i]" => $newPos[$i], "new_pos[i - 1]" => $newPos[$i - 1], "new_pos" => $newPos, "entity_pos" => array("x" => $Entity->x, "y" => $Entity->y)), "Move"); // C'est la mme, on annule les dplacements
			}
    		
    		// Si la place est occupe...
			if($tmpPos[$newPos[$i]["x"].$newPos[$i]["y"]]) {				
			if($Entity->apm) { // Si le dplacement complet ne peut tre effectu et que L'entit le souhaite, un dplacement partiel est effectu.
				if($i == 0) {
					// Cette place est occupe, comme c'est le premier dplacement  effectuer et qu'il est impossible, on ne dplace pas L'entit.
					return Error(1003, array("new_pos[i]" => $newPos[$i], "new_pos[i - 1]" => $newPos[$i - 1], "new_pos" => $newPos, "entity_pos" => array("x" => $Entity->x, "y" => $Entity->y)), "Move");
				
					break; // On arrte la boucle (au cas-o le 'return' ne suffit pas)
				} elseif($i > 0) {
					// On calcule le cout d'un dplacement
					$tDep = ($cout * $i);
							
					// On vrifie que le perso a assez de mouvements disponibles
					if($tDep > $Entity->t_mvt) { return Error(1006, array("dep_type" => "simple", "new_pos" => $newPos, "cout_dep" => $tDep, "mvt_dispo" => $Entity->t_mvt), "Move"); break; }
						
					// Cette place est occupe, mais la prcdente tait libre, on va donc placer L'entit sur cette dernire
					$newX = $newPos[$i - 1]["x"]; // Nouvelle position (x)
					$newY = $newPos[$i - 1]["y"]; // Nouvelle position (y)
						
					break; // On arrte la boucle (au cas-o le 'return' ne suffit pas)
				}
			} else {
				return Error(1005, array("dep_type" => "multi", "new_pos" => $newPos, "entity_pos" => array("x" => $Entity->x, "y" => $Entity->y)), "Move");
			}
		}
				
		$i++;
		}
			
		if(!isset($newX) && !isset($newY)) {
			// Tout a t parcouru et toutes les places sont libres, on place L'entit sur sa position finale
				
			// On calcule le cot total des dplacements de l'entit
			$tDep = ($cout * $nbMoves);
				
			// On vrifie que le perso a assez de mouvements disponibles
			if($tDep > $Entity->t_mvt) return Error(1006, array("dep_type" => "multi", "new_pos" => $newPos, "cout_dep" => $tDep, "mvt_dispo" => $Entity->t_mvt), "Move");
			
			$newX = $newPos[$nbMoves - 1]["x"]; // Nouvelle position (x)
			$newY = $newPos[$nbMoves - 1]["y"]; // Nouvelle position (y)
		}
	}
		
	// On dplace le joueur et on lui retranche le cot de son/ses dplacement(s)
	$Entity->x = $newX;
	$Entity->y = $newY;
	$Entity->t_mvt = ($Entity->t_mvt - $tDep);
		
	return $Entity->update();
}
?>
