<?php
/*
* GamePHP: Open-source library for game development
* Copyright (C) 2005 Jawad Stouli <jawad.xos@gmail.com>
* 
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

/**
 * N (any) dimensional map
 *
 * @package Engine
 * @author Jawad <jawad.xos@gmail.com>
 */
 
/**
 * N dimensional map
 *
 * Positions are defined by list of coordinates
 *
 * @package Engine
 * @subpackage Maps
 * @author Jawad <jawad.xos@gmail.com>
 */
class MapND extends SpecModel implements Map {
	/**
	 * Map informations
	 */
	public $mInfos;
	
	/**
	 * Last created zone (automatically stocked)
	 */
	protected $zoneBuffer;
	
	/**
	 * Default zone
	 */
	private $dzone;
	
	/**
	 * Array of coordinates
	 */
	private $coords;
	
	/**
	 * Initiate a map
	 *
	 * Not to be called directly, use global function getMap() instead.
	 *
	 * @param Array $mInfos Map informations (array took from maps table -MainBase-)
	 *
	 * @return 2DMap
	 */
	public function __construct($mInfos, $coords) {
		if(is_array($coords)) $this->coords = $coords;
		else return false;
		
		$this->mInfos = $mInfos;
		
		// Fetching map borders
		while($coord = $this->coord()) {
		  	$fields .= " MAX({$coord}) AS{$coord}max, MIN({$coord}) AS {$coord}min"
		}
		
		die($fields);
		
		$this->mInfos["coords"] = getConnection($this->mInfos["base"])->query("SELECT {$fields} FROM {table}", $this->mInfos["table"])->fetch();
		
		// On rcupre l'organisation des calques de la carte
		$this->mInfos["layers"] = unserialize($this->mInfos["layers"]);
	}
	
	function coord() {
	  	$this->tmpi++;
	  	
		if($this->tmpi > (count($this->coords) - 1)) {
		  	$this->tmpi = 0;
		  	
			return false;
		} else {		  	
			return $this->coords[$this->tmpi];
		}
	}
	
	/**
	 * Format (array of) layers
	 *
	 * Layers are formated when they are used outside of the engine.
	 * Formatting consists in associating type's id with type himself (inside of the engine, types aren't associated to save memory)
	 * For example:
	 *  * Unformatted: array(1, 2)
	 *  * Formatted: array(
	 *        0 => array("type" => "tile",
	 *                   "id" => 1),
	 *	      1 => array("type" => "object",
	 *                   "id" => 2))	 
	 *
	 * @param Array $layers Array of layers
	 *
	 * @return Array Formatted array
	 */	
	public function formatLayers($layers) {
		$olayers = array_flip($this->mInfos["layers"]);
		
		foreach($layers as $i=>$id) {
			if($id != _GAMEPHP_MAPS_NOTWALK && !is_array($id)) {
				// Layer is walkable and is not an entity
				$rlayers[$i]["type"] = $olayers[$i];
				$rlayers[$i]["id"] = $id;
			} elseif(is_array($id)) {
				// Layer is not an object or a tile
				foreach($id as $k=>$v) {
					$rlayers[$i][$k] = $v;
				}
			}
		}
		
		unset($olayers);
		
		return $rlayers;
	}
	
	/**
	 * Getting entities around a place
	 *
	 * @param Array $coords Coordinate (x/y)
	 * @param Array $ray Number of cases around the place
	 *
	 * @return Mixed Array
	 */
	public function getEntitiesAround($coords, $ray) {		
		$x = (int)$coords["x"];
		$y = (int)$coords["y"];
		
		$tmp = getConnection()->query("SELECT * FROM ".DBUtils::fieldsList(array_flip(getEntityList()))." WHERE (`x` != {$x} OR `y` != {$y}) AND (`x` >= ".($x - $ray)." AND `x` <= ".($x + $ray).") AND (`y` >= ".($y - $ray)." AND `y` <= ".($y + $ray).")"); 
		while($entity = $tmp->fetch()) {
			$Entities[] = $entity;
		}
		unset($tmp);
		
		return $Entities;
	}
	
	/**
	 * Rcuprer les calques d'une position depuis la base de donnes
	 *
	 * @param Int $x Coordonne x de la position
	 * @param Int $y Coordonne y de la position
	 * @param Bool $format Formater les calques ?
	 *
	 * @return Array
	 */
	protected function getLayersFromBase($x, $y, $format = true) {
		$posInfos = getConnection($this->mInfos["base"])->get("*", $this->mInfos["table"], array("x" => $x, "y" => $y));
		
		if($format) $layers = $this->formatLayers(unserialize($posInfos["layers"]));
		else $layers = unserialize($posInfos["layers"]);
		
		return isset($posInfos["layers"]) ? $layers : false;
	}
	
	/**
	 * Rcuprer les calques d'une position depuis une zone
	 *
	 * @param Int $x Coordonne x de la position
	 * @param Int $y Coordonne y de la position
	 *
	 * @return Array
	 */	
	protected function getLayersFromZone($coords, MapZone $zone, $format = true) {
		return ($format) ? $this->formatLayers($zone->getPositionLayers($coords)) : $zone->getPositionLayers($coords);
	}
	
	/**
	 * Obtenir le calque d'un type (Entity, Tile, Object...)
	 *
	 * @param String $type Type
	 *
	 * @return Int
	 */
	public function getTypeLayer($type) {
		return $this->mInfos["layers"][$type];
	}
	
	/**
	 * Obtenir les calques d'une position
	 *
	 * Le paramtre $zone est optionnel:
	 *   - vide: les informations seront rcupres directement depuis la base de donnes
	 *   - "buffer": les informations seront rcupres depuis la dernire zone cre avec getZone (si possible, sinon depuis la base de donnes)
	 *   - variable de type MapZone: les informations seront rcupres depuis la zone fournie en paramtre
	 *
	 * @param Array $coords Tableau de coordonnes (x, y)
	 * @param Mixed[Void, String, MapZone] $zone Zone d'o les calques doivent tre rcuprs
	 * @param Bool $format Faut-il formater les calques ? (non si l'utilisation est interne  la classe Map2D)
	 *
	 * @return Array
	 */
	public function getPositionLayers($coords, &$zone = "", $format = true) {
		if(!$this->positionIn($coords)) return false;
		if(empty($zone) && isset($this->dzone)) $zone =& $this->dzone;
		
		$x = (int)$coords["x"];
		$y = (int)$coords["y"];
		
		if(empty($zone)) {
			// On va aller chercher les informations depuis la base de donnes
			return $this->getLayersFromBase($x, $y, $format);
		} elseif($zone == "buffer") {
			if(isset($this->zoneBuffer)) return $this->getLayersFromZone($coords, $this->zoneBuffer); // On va aller chercher les informations depuis la zone enregistre (dernier appel  getZone)
			else return $this->getLayersFromBase($x, $y, $format); // On va aller chercher les informations depuis la base de donnes
		} else {
			// On va aller chercher les informations depuis la zone fournie en paramtre
			return $this->getLayersFromZone($coords, $zone, $format);
		}
	}
	
	/**
	 * Crer une zone
	 *
	 * Cette fonction permet de d'obtenir les informations des positions de la zone depuis la base de donnes.
	 *
	 * @param Array $coords Tableau de coordonnes (x1, x2, y1, y2)
	 *
	 * @return MapZone
	 */
	public function &getZone($coords) {		
		$zone = new Zone2D($this, $coords);
		$zone->cases = array();
		
		// On va rcuprer les informations contenues dans la zone et on les ordonne
		$tmp = getConnection($this->mInfos["base"])->query("SELECT * FROM {table} WHERE `x` >= '{$zone->x["1"]}' AND `x` <= '{$zone->x["2"]}' AND `y` >= '{$zone->y["1"]}' AND `y` <= '{$zone->y["2"]}' ORDER BY x {$zone->order}", $this->mInfos["table"]); 
		
		while($position = $tmp->fetch()) {
			$zone->setPosition(array("x" => $position["x"], "y" => $position["y"]), unserialize($position["layers"]));
		}
		
		unset($tmp);
		
		$this->zoneBuffer =& $zone;
		
		return $zone;
	}
	
	/**
	 * Check if an entity can walk on a case.
	 *
	 * @param Array $coords Array of coordinates (x, y)
	 * @param MapZone $zone Instance of MapZone (here, of Zone2D) -not compulsory, depends if you have a zone or not-
	 *
	 * @return Bool
	 */
	public function isWalkable($coords, &$zone = "") {		
		$layers = $this->getPositionLayers($coords, $zone, false);
		
		if($layers[$this->getTypeLayer("entity")] == "nw") return false;
		else return true;	  
	}
	
	/**
	 * Vrifier qu'une position existe sur la carte ( xmin <= x <= xmax et ymin <= y =< ymax)
	 *
	 * @param Array $coords Tableau de coordonnes (x/y)
	 *
	 * @return Bool
	 */
	public function positionIn($coords) {
		$x = (int)$coords["x"];
		$y = (int)$coords["y"];
		
		if(($x >= $this->mInfos["coords"]["xmin"] && $x <= $this->mInfos["coords"]["xmax"]) && ($y >= $this->mInfos["coords"]["ymin"] && $x <= $this->mInfos["coords"]["ymax"])) {
			return true;
		} else {
			return false;
		}
	}
	
	/**
	 * Dfinir un calque d'une position dans la base de donnes
	 *
	 * Cette fonction permet de dfinir les calques un par un.
	 *
	 * @param Int $layer Identifiant du calque (compris entre 1 et le nombre maximum dfinit dans la table maps)
	 * @param Array $coords Tableau de coordonnes (x, y)
	 * @param String $type Type du calque (Tile, Object, Event...)
	 * @param Int $id Identifiant unique du type associ
	 *
	 * @return Mixed[Error,Bool]
	 */
	public function setPositionLayer($coords, $type, $id) {
		if(!$this->positionIn($coords)) return false;
		
		$x = (int)$coords["x"];
		$y = (int)$coords["y"];
		
		// On vrifie sur quel calque il faut poser ce type
		$layer = $this->getTypeLayer($type);
		
		// On rcupre les informations sur cette case depuis la base
		$cInfos = $this->getLayersFromBase($x, $y, false);
		
		// A faire: vrifier si le calque est un objet ou quelque chose qu'il ne faut pas effacer
		$cInfos[$layer] = $id;
		
		// On remet l'info dans la base
		return getConnection($this->mInfos["base"])->update(array("layers" => $cInfos), array("x" => $x, "y" => $y), $this->mInfos["table"]);
	}
	
	/**
	 * Enlever le formatage des calques (vrification est effectue pour tre sr que les calques sont formats)
	 *
	 * Les calques sont formats lorsqu'ils sont exploits  l'extrieur de la classe.
	 * Lorsqu'ils sont formats, on associe  l'id du type, le type lui-mme.
	 *
	 * @param Array $layers Tableau de calques
	 *
	 * @return Array Tableau non format
	 */
	private function unformatLayers($layers) {
		$f = false;
		
		// On va d'abord vrifier que les calques sont formats
		foreach($layers as $i=>$layer) {
			if(isset($layer["type"])) { $f = true; }
			
			$newlayers[$i] = $layer["id"];
		}
		
		if(!$f) return $layers; // Non formats
		else return $newlayers;
	}
	
	/**
	 * Set a zone to be used when $zone is empty.
	 *
	 * @param MapZone $zone
	 *
	 * @since 0.1
	 */
	public function useDefaultZone(MapZone &$zone) {
		$this->dzone =& $zone;
	}
}

/**
 * Zone d'une carte  deux dimensions
 *
 * Permet de travailler sur une certaine zone d'une carte afin de ne pas surcharger le serveur en requtes.
 *
 * @package Engine
 * @subpackage Maps
 * @author Jawad <jawad.xos@gmail.com>
 */
class Zone2D implements MapZone {
	/**
	 * Variables de classe
	 */
	public $map, $x, $y, $xorder, $yorder, $cases;

	/**
	 * Crer une zone ( ne pas appeler directement, utiliser Map::getZone
	 *
	 * @param Array $coords Tableau de coordonnes (x1, x2, y1, y2)
	 * @param Int $nlayers Nombre de calques de la zone
	 *
	 * @return MapZone
	 */
	public function __construct(Map2D &$map, $coords) {
		$x1 = (int)$coords["x1"];
		$x2 = (int)$coords["x2"];
		$y1 = (int)$coords["y1"];
		$y2 = (int)$coords["y2"];
		
		if($x1 == $x2 && $y1 == $y2) return false;
		
		// On note l'ordre choisi par le dveloppeur (pour que l'itrateur le respecte)
		$this->xorder = ($x1 < $x2) ? "ASC" : "DESC";
		$this->yorder = ($y1 < $y2) ? "ASC" : "DESC";
		
		// Mais on force un ordre croissant pour nos calculs  nous (pour ne pas avoir  ordonner les requtes et autre)
		$this->x[1] = ($x1 < $x2) ? $x1 : $x2;
		$this->x[2] = ($x1 < $x2) ? $x2 : $x1;
		$this->y[1] = ($y1 < $y2) ? $y1 : $y2;
		$this->y[2] = ($y1 < $y2) ? $y2 : $y1;
		
		$this->map =& $map;
	}
	
	/**
	 * Vrifier qu'une position appartient bien  la zone
	 *
	 * @param Array $coords Tableau de coordonnes (x, y)
	 *
	 * @return Bool
	 */
	public function positionIn($coords) {
		$x = (int)$coords["x"];
		$y = (int)$coords["y"];
		
		// On vrifie si les informations sur les cases ont t rcupres
		if(isset($this->cases)) {
			// Les infos sont l, on vrifie simplement si la case demande existe
			return isset($this->cases[$x][$y]);
		}
		
		// Les infos ne sont pas l, on va s'arranger autrement
		// On vrifie si la position est dans la zone
		if(($x <= $this->x[1] && $x >= $this->x[2]) & ($y <= $this->y[1] && $y >= $this->y[2])) {
			return true;
		}
		
		return false;
	}
	
	/**
	 * Obtenir les calques d'une position (champ `layers` de la table map)
	 *
	 * @param Array $coords Tableau de coordonnes (x, y)
	 * @param Array $format Calques  formater (true, si appel direct de cette fonction)
	 *
	 * @return Array
	 */
	public function getPositionLayers($coords, $format = false) {
		$x = (int)$coords["x"];
		$y = (int)$coords["y"];
		
		if(!$this->positionIn(array("x" => $x, "y" => $y))) return false;
		
		return ($format) ? $this->map->formatLayers($this->cases[$x][$y]) : $this->cases[$x][$y];
	}
	
	/**
	 * Dfinir les calques d'une position (n'intervient pas sur la base de donnes!)
	 *
	 * Voir Map::setPositionLayer pour dfinir les calques d'une position dans la base de donnes.
	 *
	 * @param Int $x Coordonne x de la position
	 * @param Int $y Coordonne y de la position
	 * @param Array $layers Calques
	 *
	 * @return Void
	 */
	public function setPosition($coords, $layers) {
		$x = (int)$coords["x"];
		$y = (int)$coords["y"];
		
		$this->cases[$x][$y] = $layers;
	}
	
	/**
	 * Dfinir la position d'une entit
	 *
	 * @param Array $coords Tableau de coordonnes
	 * @param Int Identifiant de l'entit
	 * @param String Image
	 *
	 * @return Void
	 */
	public function setPositionEntity($coords, $id, $img) {
		$x = (int)$coords["x"];
		$y = (int)$coords["y"];
		
		$layer =& $this->cases[$x][$y][$this->map->getLayerType("entity")];
		
		$layer["type"] = "entity";
		$layer["id"] = $id;
		$layer["img"] = $img;
	}
}
?>