<?php
/*
* GamePHP: Open-source library for game development
* Copyright (C) 2005 Jawad Stouli <jawad.xos@gmail.com>
* 
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

/**
 * Fonctions d'accs aux bases de donnes (utilise PDO).
 * Non utilis pour le moment!
 *
 * @package Engine
 * @subpackage Utils
 * @author Jawad <jawad.xos@gmail.com>
 */

/**
 * Compteur de requtes
 */
$reqcounter = 0;

/**
 * Accder  la base de donnes.
 *
 * Cette classe contrle les accs  la base de donnes (connexion/dconnexion) et propose un certain nombre de fonctions simplifiant l'utilisation de MySQL.
 *
 * Exemple d'utilisation:
 * <code>
 * DB::query("requte SQL");
 * </code>
 *
 * @package Engine
 * @subpackage Utils
 * @author Jawad <jawad.xos@gmail.com>
 */
class Database {
	/**
	 * Pointeur sur la connexion.
	 *
	 * @var Resource
	 */
    private $conn;
    
    /**
     * Informations sur la connexion
     */
    private $conninfos;
    
    /**
     * Dernire requte excute
     */
    public $lastreq;
    
    /**
     * Mode SQLite ? (enlve les '`' dans une requte)
     */
    private $modelite;
    
    /**
     * Mode de test (les requtes ne sont pas excutes mais simplement gnres
     */
    static $testmode = false;
    
    /**
     * Organisation des donnes
     */
    static $fetchmode = PDO_FETCH_ASSOC;
    
    /**
     * Se connecter au serveur de bases de donnes.
     *
     * @param String Serveur
     * @param String Nom d'utilisateur
     * @param String Mot de passe
     * @param String Base de donnes
     *
     * @return Void
     */
    public function __construct($db = "MainBase") {
    	Global $GamePHP;
    	
    	$this->conninfos =& $GamePHP[$db];
    	
    	$dsn = (empty($this->conninfos["Type"]) == true) ? $this->conninfos["DSN"] : ($this->conninfos["Type"].":dbname=".$this->conninfos["Name"].";host=".$this->conninfos["Host"]);
    
    	$this->modelite = (strpos($dsn, "sqlite") !== false) ? true : false;
    
    	$this->conn = new PDO($dsn, $this->conninfos["User"], $this->conninfos["Password"]);
    }
    
    /**
     * Compter le nombre d'enregistrement.
     *
     * @param String Table
     * @param String Clauses (fin de la requte SQL: WHERE...)
     *
     * @return Numeric Nombre d'enregistrement retourn  l'excution de la requte
     */
	public function count($table, $clauses = "") {
		$rs = $this->query($this->formatSQL("SELECT COUNT(*) AS nb FROM {table} ".$clauses, $table))->fetch();
		
		return $rs["nb"];
	}
	
	/**
	 * Vrifie qu'un enregistrement existe.
	 *
	 * @param String Champs
	 * @param String Valeur
	 *
	 * @return Bool
	 */
	public function exists($datas, $table) {	
		$nb = $this->count($table, "WHERE ".DBUtils::datasList($datas, "AND ", $encode)." LIMIT 0,1");
		
		if($nb == 0) return false;
		else return true;
	}
	
	/**
	 * Rcupre une ligne depuis la base de donnes
	 *
	 * @param Array $fields Champs  rcuprer
	 * @param String $table Table
	 * @param Array $where Clause where
	 *
	 * @return Resource PDOStatement
	 */
	public function get($fields, $table, $where) {	
		return $this->query("SELECT {$fields} FROM {table} WHERE ".DBUtils::datasList($where, " AND ")." LIMIT 0,1", $table)->fetch();
	}
	
	/**
	 * Effectue une requte Insert
	 *
	 * @param Array $arr Tableau de valeurs
	 * @param String $table Table
	 * @param Bool $encode Faut-il encoder les valeurs ?
	 *
	 * @return Rsultat de la requte (Objet MySQLi_RS)
	 */
	public function insert($arr, $table, $encode = true) {
		$SQL = "INSERT INTO {table} (".DBUtils::fieldsList($arr).") VALUES (".DBUtils::valuesList($arr, $encode).")";
		
		return $this->query($this->formatSQL($SQL, $table));
	}
    
    /**
     * Formate une requte SQL (remplace {table} par la table correspondante)
     *
     * @param String Requte SQL
     * @param String Table
     *
     * @return String Requte formate
     */
    public function formatSQL($SQL, $table = "") {    	
    	if(!empty($table) && isset($this->conninfos["Tables"][$table]["name"])) $SQL = str_replace("{table}", "`".$this->conninfos["Tables"][$table]["name"]."`", $SQL);
    	if($this->modelite) $SQL = str_replace("`", "", $SQL);
    	
    	return $SQL;
    }
    
    /**
	 * Excute une requte SQL.
	 *
	 * @param String Requte SQL
	 * @param String Table
	 *
	 * @return Rsultat de la requte (Objet MySQLi_RS)
	 */
    public function &query($SQL, $table = "") {
    	Global $insert_id, $reqcounter;
    
    	$SQL = $this->formatSQL($SQL, $table);
    	
    	if(!self::$testmode) {
    		$rs =& $this->conn->query($SQL);
    		if(!$rs) return Error(1, "&DB::query", __FILE__, __LINE__, array("Code d'erreur" => $this->conn->errorCode(), "Informations du serveur" => $this->conn->errorInfo(), "Requte SQL" => $SQL));
        }
		
		$this->lastreq = $SQL;
		$reqcounter++;
		
		if(get_class($rs) == "PDOStatement") $rs->setFetchMode(self::$fetchmode);
			
    	return $rs;
    }
    
	/**
	* Effectue une requte Update
	*
	* @param Array $arr Tableau  convertir
	* @param String $plus Chaine  rajouter  la fin de la requte (clause where...)
	*
	* @return String
	*/
	function update($arr, $where, $table, $encode = true) {
		$SQL = "UPDATE {table} SET ".DBUtils::datasList($arr, ", ", $encode)." WHERE ".DBUtils::datasList($where, " AND ", $encode)." ".$plus;
		
		return $this->query($SQL, $table);
	}
}

/**
 * Fonction permettant d'obtenir une connexion avec la base de donnes
 *
 * La connexion est cre si elle n'existe pas dj.
 *
 * @param String $db Nom de la base de donnes  laquelle on tente d'accder.
 *
 * @return Database
 */
function &getConnection($db = "MainBase") {
	if(isset($GLOBALS["conn_".$db])) {
		return $GLOBALS["conn_".$db];
	} else {
		$GLOBALS["conn_".$db] =& new Database($db);
	
		return $GLOBALS["conn_".$db];
	}
}

/**
 * Fonctions gnrales de traitement des requtes SQL.
 *
 * @package Engine
 * @subpackage Utils
 * @author Jawad <jawad.xos@gmail.com>
 */
abstract class DBUtils {
	/**
	 * Transforme un tableau en partie de requte SQL (pour les clauses WHERE et requtes UPDATE)
	 *
	 * @param Array Tableau de valeurs (cl = 'valeur')
	 * 
	 * @return String
	 */
	public function datasList($datas, $sep, $encode = true) {
		if(count($datas) == 0) return 1;
		
		$rVal = array();
	
		foreach($datas as $key=>$value) {
			if($encode) {
				if(is_array($value) || is_object($value)) {
		   			$rVal[] = "`{$key}` = '".serialize($value)."'";
				} else {                                        
					$rVal[] = "`{$key}` = '".self::encode($value)."'";
				}
			} else {
				$rVal[] = "`{$key}` = '".$value."'";
			}
		}
		
		return implode($sep, $rVal);
	}
	
	/**
	 * Dcode une chaine provenant de la base de donnes pour son affichage (si encode pralablement).
	 *
	 * Si vous voulez interdire l'interprtation du HTML, ne dcodez pas la chaine!
	 *
	 * @param String Chaine  dcoder.
	 * @param Constant Dcoder les guillemets
	 *
	 * @return Void
	 * @link http://www.php.net/html_entity_decode
	 */
	public function decode($string, $quotes = ENT_QUOTES) {   
    	return str_replace("<br />", "", html_entity_decode($string, $quotes));
	}
	
	/**
	 * Encode une variable avant de l'exploiter dans une requte SQL.
	 *
	 * @param String/Array Variable  encoder.
	 *
	 * @return Void
	 */
	public function encode($var) {
		if(is_numeric($var)) {
			return $var;
		} elseif(is_string($var)) {
			return nl2br(htmlentities($var, ENT_QUOTES));
		} elseif(is_array($var)) {
			foreach($var as $key=>&$value) {
				if(is_array($value) || is_object($value)) {
					$value = serialize($value);
				} else {
					$value = nl2br(htmlentities($value, ENT_QUOTES));
				}
				
			}
			
			return $var;
		}	
		
		return false;
	}
	
	/**
	 * Transforme un tableau en liste de champs.
	 *
	 * @param Array $table Tableau  convertir
	 *
	 * @return String
	 */
	public function fieldsList($arr) {
		foreach($arr as $f=>$v) {
			$fields[] = "`".$f."`";
		}
	
		return implode(",", $fields);
	}
	
	/**
	 * Obtenir le nom d'un champ sur la base de donnes
	 *
	 * @param String $field Nom interne du champ
	 * @param String $table Nom interne de la table
	 * @param String $base Nom interne de la base de donnes
	 *
	 * @return String
	 */
	public function getField($field, $table, $base = "MainBase") {
		Global $GamePHP;
		
		if(isset($GamePHP[$base]["Tables"][$table]["field_{$field}"])) return $GamePHP[$base]["Tables"][$table]["field_{$field}"];
		else return "null";
	}
	
	/**
	 * Transforme un tableau en liste de valeurs.
	 *
	 * @param Array $table Tableau  convertir
	 *
	 * @return String Liste de valeurs.
	 */
	public function valuesList($arr, $encode = true) {
		foreach($arr as $f=>$v) {
			if($encode) $v = self::encode($v);
		
			$values[] = "'".$v."'";
		}
	
		return implode(",", $values);
	}
}
?>
