<?php
/*
* GamePHP: Open-source library for game development
* Copyright (C) 2005 Jawad Stouli <jawad.xos@gmail.com>
* 
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

/**
 * Fonctions gnralistes.
 * 
 * Un ensemble de fonctions utilises par de nombreux fichiers.
 * Certaines fonctions sont utiles pour votre dveloppement.
 *
 * @package Engine
 * @subpackage Utils
 * @author Jawad <jawad.xos@gmail.com>
 */
 
 /**
 * Obtenir la distance entre deux cases.
 *
 * Aucune vrification n'est faite pour vrifier que les deux cases n'en font pas qu'une seule (utilisez caseTest avant d'appeller cette fonction, afin d'viter un traitement inutile).
 *
 * @param array $pos1 Position de la case 1 - array("x" => x1, "y" => y1) -
 * @param array $pos2 Position de la case 2 - array("x" => x2, "y" => y2) -
 *
 * @return Int Distance
 */
function caseDiff($pos1, $pos2) {
 	$test = caseTest($pos1, $pos2);
 	
 	if($test == 2) {
 		// Les cases ne sont ni les mmes ni colles, on va devoir effectuer le calcul
 	} else {
 		return $test;
	}
}
 
 /**
 * Obtenir la position relative d'une case par rapport  une autre (mme case, cases colles, cases non-colles).
 *
 * @param array $pos1 Position de la case 1 - array("x" => x1, "y" => y1) -
 * @param array $pos2 Position de la case 2 - array("x" => x2, "y" => y2) -
 *
 * @return Int 0: Mme case, 1: cases colles, 2: cases loignes
 */
function caseTest($pos1, $pos2) {
    if(($pos1["x"] == $pos2["x"]) || ($pos1["x"] == ($pos2["x"] - 1)) || ($pos1["x"] == ($pos2["x"] + 1))) {
    	// X: OK!
    	if($pos1["y"] == $pos2["y"] && $pos1["x"] == $pos2["x"]) {
    		// C'est la mme case, on retourne 0
    		return 0;
		} elseif($pos1["y"] == ($pos2["y"] - 1) || $pos1["y"] == ($pos2["y"] + 1) || ($pos1["y"] == $pos2["y"] && $pos1["x"] != $pos2["x"])) {
			// Les deux cases sont colles, on retourne 1
			return 1;
		}
	}
		
	return 2; // Les deux cases sont loignes, on retourne 2
}
 
 /**
 * Lancer un ou des ds
 *
 * Cette fonction permet d'effectuer le tirage des ds et d'en faire la somme (dans le cas de plusieurs lancs).
 * Retourne un tableau avec le rsultat de chaque d ou la somme des ds, ou encore le rsultat du lancer (lorsqu'il est unique).
 *
 * @param int $nbDe Nombre de ds  lancer
 * @param int $nbFace Nombre de faes des ds
 * @param bool $sum Faire la somme des ds ?
 *
 * @return Mixed (Int si somme, Array sinon)
 */
function diceRoll($nbDe = 1, $nbFace = 5, $sum = false) {		
	$i = $nbDes;
			
    if ($i == 1) {
    	// Un seul d  lancer, on retourne directement le rsultat
	    return mt_rand(1, $nbFace);
	} else {
		// Plusieurs ds  lancer
		$i = 1;
		$t_sum = 0;
    	while($i <= $nbDe) {
        	$de[$i] = mt_rand(1, $nbFace);
        	if ($sum) $t_sum += $de[$i]; // On fait la somme si ncessaire
        	$i++;
    	}
    		
    	if ($sum) return $t_sum; // On retourne la somme (si demand)
    	else return $de; // On retourne un tableau contenant la liste des rsultats (si la somme n'est pas demande)
	}
}

/**
 * Mettre en forme l'affichage d'une variable (surcharge de la fonction var_dump)
 *
 * @param Mixed Variable  afficher.
 *
 * @link http://www.php.net/var_dump
 */
function var_dump_pre($var) {
	echo "<pre>";
	var_dump($var);
	echo "</pre>";
}
?>