<?php
/*
* GamePHP: Open-source library for game development
* Copyright (C) 2005 Jawad Stouli <jawad.xos@gmail.com>
* 
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

/**
 * Itrateur
 *
 * @package Engine
 * @subpackage Utils
 * @author Jawad <jawad.xos@gmail.com>
 */
 
/**
 * Boucler sur des tableaux de coordonnes x/y
 *
 * @package Engine
 * @subpackage Utils
 * @author Jawad <jawad.xos@gmail.com>
 */
class MyIterator {
	/**
	 * Variables de classe
	 */
	protected $coords, $currx, $curry, $stop = false;
	public $xorder = "ASC", $yorder = "ASC";
	
	/**
	 * Gnrer les informations de retour
	 *
	 * @return Array
	 */
	protected function buildResult() { 
		return array("x" => $this->currx, "y" => $this->curry);
	}
	
	/**
	 * Instancier un compteur
	 *
	 * @param Array $coords Coordonnes (xmin, ymin, xmax, ymax)
	 *
	 * @return MyIterator
	 */
	public function __construct($coords) {	
		$this->coords = $coords;
		$this->initCoords();
	}
	
	/**
	 * Obtenir les informations de coordonnes et avancer d'un dans le dcompte
	 *
	 * @return Array
	 */
	public function fetch() {
		if($this->stop) return false;
		
		$tmp = $this->buildResult();
	
		if($this->currx == $this->coords["xmax"]) {
			// On a fini la boucle sur les x
			// On vrifie si on a fini la boucle sur les y
			if($this->curry == $this->coords["ymax"]) $this->stop = true;
			
			// Si non, on remet les x  xmin et on incrmente (ou dcrement) de 1 les y
			$this->currx = $this->coords["xmin"];
			
			if($this->yorder == "ASC") $this->curry++;
			else $this->curry--;
		} else {
			if($this->xorder == "ASC") $this->currx++;
			else $this->currx--;
		}
		
		return $tmp;
	}
	
	/**
	 * Mettre  zro le compteur
	 *
	 * @return Void
	 */
	public function initCoords() {
		$this->currx = $this->coords["xmin"];
		$this->curry = $this->coords["ymin"];
	}
	
	/**
	 * Vrifier si nous sommes  la fin d'une ligne
	 *
	 * @return Bool
	 */
	public function endLine() {
		if($this->currx == $this->coords["xmin"]) return true;
	}
	
	/**
	 * Vrifier si nous sommes dans une nouvelle ligne
	 *
	 * @return Bool
	 */
	 public function newLine() {
		if($this->xorder == "ASC" && $this->currx == $this->coords["xmin"] + 1) return true;
		elseif($this->xorder == "DESC" && $this->currx == $this->coords["xmin"] - 1) return true;
	 }
}
?>