<?php
/*
* GamePHP: Open-source library for game development
* Copyright (C) 2005 Jawad Stouli <jawad.xos@gmail.com>
* 
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

/**
 * Gestionnaire des valeurs de retour
 *
 * @package Engine
 * @subpackage Utils
 * @author Jawad <jawad.xos@gmail.com>
 */
 
/**
 * Crer une valeur de retour
 *
 * @package Engine
 * @subpackage Utils
 * @author Jawad <jawad.xos@gmail.com>
 * @see Globals.php
 */
class Result {
	/**
	 * Variables
	 */
	public $code, $func, $debug, $msg, $file, $line;
	
	/**
	 * Faut-il automatiquement afficher les erreurs ?
	 */
	static $autoraise;
	
	/**
	 * Constructeur
	 *
	 * @param Int Code de retour (-2 = Erreur, -1 = Avertissement -l'opration a chou mais cela n'empche pas l'excution du moteur-
	 * @param String Message de retour (peut-tre un identifiant dfini dans Globals.php)
	 * @param String Fonction o est apparue l'erreur
	 * @param String Fichier o est apparue l'erreur
	 * @param Int Ligne o est apparue l'erreur
	 * @param Mixed Informations de dbuggage
	 */
	public function __construct($code, $sz, $func = "", $file = "", $line = "", $debug = "") {
		Global $Results;
		
		$this->code = $code;
		$this->msg = (is_numeric($sz) && isset($Results[$sz])) ? $Results[$sz] : $sz;
		$this->func = $func;
		$this->file = $file;
		$this->line = $line;
		$this->debug = $debug;

		if(self::$autoraise && $this->isError()) $this->getError(true);		
	}
	
	/**
	 * Faire remonter l'erreur.
	 *
	 * Par dfaut, l'appel de cette fonction provoque l'affichage de l'erreur.
	 *
	 * @param Bool Faut-il afficher les informations de dbuggage ?
	 *
	 * @return Void
	 */
	public function getError($debug = false) {
		echo "<div id=\"GamePHP_Error\"><b>Une erreur a t souleve.</b><br />Code d'erreur: {$this->code}<br />Message: {$this->msg}<br />Fonction: {$this->func}<br />Fichier: {$this->file}<br/>Ligne: {$this->line}<br />Informations de dbuggage:<br />";
		if($debug) var_dump_pre($this->debug);	
		echo "</div>";
	}
}

/**
 * Retourner une erreur
 *
 * @param $sz String Chane d'erreur
 * @param $func String Fonction ayant choue
 * @param $file String Fichier contenant la fonction
 * @param $line String Ligne d'erreur
 * @param $debug Mixed Informations de dbuggage
 *
 * @return Result
 */
function &Error($sz, $func = "", $file = "", $line = "", $debug = "") {
	return new Result(-2, $sz, $func = "", $file = "", $line = "", $debug = "");
}

/**
 * Retourner un avertissement
 *
 * @param $sz String Message d'avertissement
 * @param $func String Fonction ayant choue
 * @param $file String Fichier contenant la fonction
 * @param $line String Ligne d'erreur
 * @param $debug Mixed Informations de dbuggage
 *
 * @return Result
 */
function &Warning($sz, $func = "", $file = "", $line = "", $debug = "") {
	return new Result(-1, $sz, $func = "", $file = "", $line = "", $debug = "");
}

/**
 * Retourner un succs
 *
 * @param $sz String Message de russite
 *
 * @return Result
 */
function &Success($sz) {
	return new Result(0, $sz);
}

/**
 * Vrifier si une variable est une erreur
 *
 * Permet de vrifier si le rsultat d'une fonction est une erreur.
 *
 * @param $var Result Objet Result
 *
 * @return Bool
 */
function isError($var) {
	if(get_class($var) == "Result" && $var->code == -2) return true;
}

/**
 * Vrifier si une variable est un avertissement
 *
 * Permet de vrifier si le rsultat d'une fonction est un avertissement.
 *
 * @param $var Result Objet Result
 *
 * @return Bool
 */
function isWarning($var) {
	if(get_class($var) == "Result" && $var->code == -1) return true;
}

/**
 * Vrifier si une variable est un succs
 *
 * Permet de vrifier si le rsultat d'une fonction est un succs.
 *
 * @param $var Result Objet Result
 *
 * @return Bool
 */
function isSuccessful($var) {
	if((get_class($var) == "Result" && $var->code == 0) || (!isError($var) && !isWarning($var))) return true;
}
?>