<?php
/*
* GamePHP: Open-source library for game development
* Copyright (C) 2005 Jawad Stouli <jawad.xos@gmail.com>
* 
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

/**
 * Fichier d'action (Attack)
 *
 * Cette action permet  une entit Player d'attaquer une autre entit Player.
 *
 * @package Engine
 * @subpackage Actions
 */
 
$Actions["Attack"]["entities"] = array("Player"); // Seule l'entit 'Player' peut effectuer cette action
$Actions["Attack"]["params"] = array("Cible");
$Actions["Attack"]["errors"] = array(
	0 => "L'attaque a russi, vous perdez 2 points d'action et votre adversaire perd 5 points de vie.",
	1 => "L'attaque a chou, vous perdez 2 points d'action et votre adversaire gagne 5 points de vie.",
	2 => "Vous n'tes pas en position pour effectuer cette attaque.",
	3 => "Vous n'avez pas assez de point d'action pour attaquer.",
	4 => "Un problme technique empche votre attaque.",
);

/**
 * Dplacer une entit
 *
 * @param Entity Entit  dplacer
 * @param Array Paramtres de dplacement - array("x" => nouveau x, "y" => nouveau y) -
 *
 * @example Actions/Move.php
 *
 * @return Int 0 (La place est occupe), 1 (Mise  jour des donnes impossible), 2 (Dplacement effectu)
 */
function Attack(Entity &$Entity, $params) {
	Global $Actions;
	
    if($Entity->ap_turn < 2) return new Result(3, $Actions["Attack"]["errors"][3], "Attack", __FILE__, __LINE__, array("Entit" => $Entity, "params" => $params));
	
	$Cible =& $params["Cible"]; // Il est important de conserver la rfrence, si on utilisait une copie, il y'aurait une diffrence entre l'entit fournie en paramtre lors de l'exction de l'action (dans votre code), et celle mise  jour aprs l'action.
	
	if(caseTest(array("x" => $Entity->x, "y" => $Entity->y), array("x" => $Cible->x, "y" => $Cible->y)) != 1) return new Result(2, $Actions["Attack"]["errors"][2], "Attack", __FILE__, __LINE__, array("Entit" => $Entity, "params" => $params)); // Les deux cases ne sont pas colles; attaque impossible!
	
	$Entity->ap_turn -= 2;
	
	// On effectue un lancer de d 20 pour vrifier que le joueur russit son attaque (si le score est >= 10)
	if(diceRoll(1, 20) >= 10) {
		// Attaque russie, on enlve 5 points de vie  la cible!
		$Cible->hp_turn -= 5;
		
		$ret = new Result(0, $Actions["Attack"]["errors"][0], "Attack", __FILE__, __LINE__, array("Entit" => $Entity, "params" => $params));
	} else {
		// L'attaque a chou, on rajoute 5 points de vie  la cible (chacun ses rgles hein)!
		$Cible->hp_turn += 5;
		
		$ret = new Result(1, $Actions["Attack"]["errors"][1], "Attack", __FILE__, __LINE__, array("Entit" => $Entity, "params" => $params));
	}
	
	if(!$Entity->update()) $ret = new Result(4, $Actions["Attack"]["errors"][4], "Attack", __FILE__, __LINE__, array("Entit" => $Entity, "params" => $params));
	if(!$Cible->update()) {
		$Entity->rollback();
		$ret = new Result(4, $Actions["Attack"]["errors"][4], "Attack", __FILE__, __LINE__, array("Entit" => $Entity, "params" => $params));
	}
	
	return $ret;
}
?>