<?php
/*
* GamePHP: Open-source library for game development
* Copyright (C) 2005 Jawad Stouli <jawad.xos@gmail.com>
* 
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

/**
 * Objet de base
 *
 * @package Engine
 * @subpackage Objects
 * @author Jawad <jawad.xos@gmail.com>
 */


/**
 * Objet de base ( ne pas utiliser normalement)
 *
 * @package Engine
 * @subpackage Objects
 * @author Jawad <jawad.xos@gmail.com>
 */
class Object {
	/**
	 * Caractristiques de l'objet
	 */
	public $oCaracs;
	
	/**
	 * Qui peut utiliser ce type d'objet ? (false = tous, sinon, tableau contenant une liste d'entit)
	 */
	protected $use = false;

	/**
	 * L'entit peut-elle exploiter cet objet ?
	 *
	 * @param Entity $Entity Entit voulant exploiter cet objet
	 *
	 * @return bool
	 */
	public function canUse(Entity $Entity) {	
		if($this->use === false) {
			return true;
		} elseif(in_array(get_class($Entity), $this->use)) {
			return true;
		} else {
			return false;
		}
	}

	/**
	 * Chargement des informations de l'objet
	 *
	 * @param Int $objectId Identifiant de l'objet
	 * @return Void
	 */
	public function __construct($oCaracs) {
		$this->oCaracs = $oCaracs;
	}
	
	/**
	 * Enlever l'objet  une entit.
	 *
	 * @param Object $Object Objet  enlever
	 * @param Entity $Entity Entit  qui il faut enlever l'objet.
	 *
	 * @return Bool
	 */
	public function _drop(Entity &$Entity) {
		if(!$this->canUse($Entity)) return false;
	
		$objfield = DBUtils::getField("objects", $Entity->table);
		
		if(!in_array($this->oCaracs["id"], $Entity->{$objfield})) return false;
		
		$dropped = false;
		foreach($Entity->{$objfield} as $key=>$value) {
			if(!$dropped && $value == $this->oCaracs["id"]) $dropped = true;
    		else $newtmp[] = $value;
		}
		
		$Entity->{$objfield} = $newtmp;
		$Entity->update();
		
		unset($newtmp);
		
		return true;
	}

	/**
	 * Donner un objet  une entit
	 *
	 * @param Object Objet  donner - obtenu avec ObjectManager::get() -
	 * @param Entity Entit  qui l'objet doit tre donn
	 *
	 * @return Bool
	 */
	public function _give(Entity &$Entity) {
		if(!$this->canUse($Entity)) return false;
		
		$objfield = DBUtils::getField("objects", $Entity->table);
		
		// On ajoute l'objet  la liste des objets de l'entit
		// Impossible d'utiliser update si vous avez modifi $Entity->{$objfield}[]
		// D'o l'emploi d'une variable temporaire $tmp
		$tmp = $Entity->{$objfield};
		$tmp[] = $this->oCaracs["id"];
		$Entity->{$objfield} = $tmp;
		
		unset($tmp);
		
		$Entity->update(); // On met  jour les informations de l'utilisateur
		
		return true;
	}
	
	/**
	 * Fonction appele lorsque cet objet est jet au sol par une entit.
	 *
	 * @param Array $infos Informations sur l'emplacement o est jet l'objet
	 * @param Entity $Entity Entit jetant l'objet.
	 *
	 * @return Mixed
	 */
	public function _throw(Map $Map, $coords, Entity &$Entity) {
		if(!$this->canUse($Entity)) return false;
		
		$this->_drop($Entity); // On enlve l'objet  l'entit
		
		$Map->setPositionLayer("", $coords, "object", $this->oCaracs["id"]);
	}
	
	/**
	 * Fonction appele lorsque cet objet est utilis
	 *
	 * @param $Entity Entit utilisant l'objet
	 *
	 * @return Mixed
	 */
	public function _use(Entity &$Entity) {
		if(!$this->canUse($Entity)) return false;
	
		// Cette objet n'a aucun effet!
	}
}
?>